#include "circle.h"
#include "aabb.h"

void Circle::Rasterize(Image &image) {

  // figure out axis aligned bounding box AABB of circle
  AABB aabb(center+V2D(+r, +r));
  aabb.AddPoint(center+V2D(-r, -r));

  // traverse AABB and set pixels if inside circle and inside image frame
  for (int ri = aabb.TopRow(); ri <= aabb.BottomRow(); ri++) {
    for (int ci = aabb.LeftColumn(); ci <= aabb.RightColumn(); ci++) {
      V2D pixelCenter(.5f+(float)ri, .5f+(float)ci);
      if ((pixelCenter-center).Length() > r)
        continue;
      image.SetPixelIfInsideFrame(ri, ci, color);
    }
  }

}

Circle::Circle(V2D _center, float _r, Color _color) : center(_center), 
  r(_r), color(_color) {}

Circle::Circle(istream &istr) {

  bool readCenter = false;
  bool readRadius = false;
  color = Color((char)219);
  while (1) {
    char keyword[100];
    istr >> keyword;
    if (istr.fail()) {
      cerr << "ERROR: while reading circle" << endl;
      return;
    }
    if (!strcmp(keyword, "endOfCircle")) {
      break;
    }
    else if (!strcmp(keyword, "center")) {
      istr >> center;
      readCenter = true;
    }
    else if (!strcmp(keyword, "radius")) {
      istr >> r;
      readRadius = true;
    }
    else if (!strcmp(keyword, "color")) {
      istr >> color;
    }
  }
  if (!readCenter || !readRadius) {
    cerr << "ERROR: did not read all required circle data" << endl;
  }

}