#include "image.h"

ostream& operator<<(ostream &ostr, Color color) {

  ostr << color.color;
  return ostr;

}

ImageRow::ImageRow(int _w) {

  Allocate(_w);

}

void ImageRow::Allocate(int _w) {

  w = _w;
  colors = new Color[w];

}

ImageRow::~ImageRow() {

  delete []colors;

}

Color& ImageRow::operator[](int i) {

  return colors[i];

}

ostream& operator<<(ostream& ostr, ImageRow &imageRow) {

  for (int i = 0; i < imageRow.w; i++) {
    ostr << imageRow[i];
  }

  return ostr;

}

Image::Image(int _w, int _h) : w(_w), h(_h) {

  rows = new ImageRow[h];
  for (int ri = 0; ri < h; ri++) {
    rows[ri].Allocate(w);
  }

}

void Image::Show() {

  Show(cerr);

}

void Image::Show(ostream &ostr) {

  for (int ri = 0; ri < h; ri++) {
    ostr << rows[ri] << endl;
  }

}

void Image::MakeChecker(int checkerSize, Color col0, Color col1) {

  for (int ri = 0; ri < h; ri++) {
    int checkerRowi = ri/checkerSize;
    for (int ci = 0; ci < w; ci++) {
      int checkerColumni = ci/checkerSize;
      Color colorToUse = col0;
      if ((checkerRowi+checkerColumni)%2)
        colorToUse = col1;
      rows[ri][ci] = colorToUse;
    }
  }

}

ImageRow& Image::operator[](int ri) {

  return rows[ri];

}

Image::~Image() {

  delete []rows;

}

void Image::Clear(Color bgColor) {

  for (int ri = 0; ri < h; ri++) {
    for (int ci = 0; ci < w; ci++) {
      (*this)[ri][ci] = bgColor;
    }
  }

}

bool Image::InsideFrame(int ri, int ci) {

  return (ri >= 0) && (ri < h) && (ci >=0) && (ci < w);

}

void Image::SetPixelIfInsideFrame(int ri, int ci, Color color) {

  if (!InsideFrame(ri, ci))
    return;
  (*this)[ri][ci] = color;

}

istream& operator>>(istream &istr, Color& col) {

  int intColor;
  istr >> intColor;
  col = Color(intColor);
  return istr;

}


void Image::Save(char *fname) {

  ofstream ofs(fname);
  Show(ofs);
  ofs.close();

}